import binascii
import os
import time

import json

try:
    import pg_util
    import trials
except ImportError:
    from . import pg_util
    from . import trials

try:
    from odoo.release import serie
except ImportError:
    serie = "16.0"

try:
    with open(os.path.expanduser("~/config/config.json")) as c:
        saas_config = json.load(c)
except (IOError, ValueError):
    saas_config = {}

DEMO = "demo" in saas_config.get("me", {}).get("mode", "").split()
MAX_DEMO_DATABASES = 2000
MAX_DEMO_LIFETIME = 60 * 60
DEMO_LANGUAGES = ["en_US"]
DEMO_ADMIN_PASS = "admin"

DB_PREFIX = serie.replace(".", "").replace("~", "-")

DEMO_COMMON_PREFIX = "demo_"
DEMO_PREFIX = "{0}{1}_".format(DEMO_COMMON_PREFIX, DB_PREFIX)


def demo_dbname():
    instance_id = binascii.hexlify(os.urandom(6)).decode()
    return "{}{}_{}".format(DEMO_PREFIX, instance_id, int(time.time()))


def demo_list(all_versions=False):
    prefix = DEMO_COMMON_PREFIX if all_versions else DEMO_PREFIX
    query = "SELECT datname FROM pg_database WHERE datname like %s"
    with pg_util.cursor("postgres") as cr:
        query = cr.mogrify(query, [prefix.replace("_", r"\_") + "%"])
        cr.execute(query)
        demo_dbs = [db[0] for db in cr.fetchall()]

    return demo_dbs


def demo_time(dbname):
    """return the time the given demo database was created"""
    return int(dbname.rpartition("_")[-1])


def demo_gc():
    """drop demo databases that are too old, whatever the version"""
    deadtime = time.time() - MAX_DEMO_LIFETIME
    for dbname in demo_list(all_versions=True):
        if demo_time(dbname) < deadtime:
            trials.drop(dbname)
